




// START OF GET_FRIVE_TYPE() IMPLEMENTATION


/*
  Include file    get_drive_type.inc
  Author          Carlo Hogeveen
  Website         ecarlo.nl/tse
  Version         v2.0.0.-1   (in development)
  Compatibility   Untested

  This implementation provides the wrapper function get_drive_type().

  Unlike the Windows function GetDriveType(), which only accepts root(drive)s,
  get_drive_type() also accepts directory and file paths,
  as long as they start with their root (drive).

  get_drive_type() also accepts directories and files on shares,
  and likewise determines their roots and "drive types".
  For example: "\\Share\", "\\Server\Share", "\\Server\Share\secrets.txt".

  An intentional difference:
    Windows' GetDriveType("\\") returns DRIVE_REMOTE.
    My     get_drive_type("\\") returns DRIVE_UNKNOWN.
  Here it is more useful to treat "\\" as an invalid root, because otherwise
  non-existent shares would appear to exist on the valid remote drive "\\".

  get_drive_type() is optimized for speed by remembering previously determined
  roots and their drive types.

  In Linux get_drive_type() always returns DRIVE_FIXED, pretending everything
  is on a fixed harddrive.


  HISTORY

  v1   11 Mar 2021
    This version was based on Dos() calls to "WMIC",
    and was only compatible with Windows TSE 4.0 upwards.

  v2.0.0.-1 (in development)   7 Nov 2023


*/


#define DRIVE_UNKNOWN     0 // The drive type cannot be determined.
#define DRIVE_NO_ROOT_DIR 1 // The root path is invalid; for example, there is no volume mounted at the specified path.
#define DRIVE_REMOVABLE   2 // The drive has removable media; for example, a floppy drive, thumb drive, or flash card reader.
#define DRIVE_FIXED       3 // The drive has fixed media; for example, a hard disk drive or flash drive.
#define DRIVE_REMOTE      4 // The drive is a remote (network) drive.
#define DRIVE_CDROM       5 // The drive is a CD-ROM drive.
#define DRIVE_RAMDISK     6 // The drive is a RAM disk.


#ifdef LINUX

  integer proc get_drive_type(string fqn_string)
    string dummy [1] = fqn_string
    dummy = dummy // To avoid compiler warning.
    return(DRIVE_FIXED)
  end get_drive_type

#else

  dll "<kernel32.dll>"
    integer proc GetDriveType(
        string      lpRootPathName              : CStrVal
    )
    : "GetDriveTypeA"
  end

  integer get_drive_type_id      = 0
  integer get_drive_type_entries = 0

  integer proc get_drive_type(string fqn_string)
    integer fqn_type                   = DRIVE_UNKNOWN
    integer root_entry                 = 0
    string  root_string [MAXSTRINGLEN] = ''

    if get_drive_type_id == 0
      PushLocation()
      get_drive_type_id = CreateTempBuffer()
      ChangeCurrFilename(SplitPath(CurrMacroFilename(), _NAME_) + ':id',
                         _DONT_PROMPT_|_DONT_EXPAND_|_OVERWRITE_)
      PopLocation()
    else
      for root_entry = 1 to get_drive_type_entries
        root_string = GetBufferStr('String' + Str(root_entry), get_drive_type_id)
        if Pos(root_string, fqn_string) == 1
          fqn_type = GetBufferInt('Type' + Str(root_entry), get_drive_type_id)
          break
        endif
      endfor
    endif

    //  Note:
    //    I noticed one network reference needing a trailing slash,
    //    so parent paths are tested with and without it,
    //    trying the longer, more specific path (with traing slash) first.

    root_string = fqn_string

    while (fqn_type in DRIVE_UNKNOWN, DRIVE_NO_ROOT_DIR)
    and   Length(root_string) > 0
      fqn_type = GetDriveType(root_string)
      if (fqn_type in DRIVE_UNKNOWN, DRIVE_NO_ROOT_DIR)
        root_string = RemoveTrailingSlash(root_string)
        fqn_type    = GetDriveType(root_string)
      endif
      if (fqn_type in DRIVE_UNKNOWN, DRIVE_NO_ROOT_DIR)
        if Pos('\', root_string)
          root_string = SplitPath(root_string, _DRIVE_|_PATH_)
        else
          root_string = ''
        endif
      else
        // GetDriveType("\\") returns drive type DRIVE_REMOTE, which might be
        // an OK Windows default, but in TSE's context it is not a valid drive.
        if root_string == '\\'
          fqn_type    = DRIVE_UNKNOWN
          root_string = ''
        else
          get_drive_type_entries = get_drive_type_entries + 1
          SetBufferStr('String' + Str(get_drive_type_entries), root_string,
                       get_drive_type_id)

          SetBufferInt('Type'   + Str(get_drive_type_entries), fqn_type,
                       get_drive_type_id)
        endif
      endif
    endwhile

    return(fqn_type)
  end get_drive_type

#endif


// END OF GET_FRIVE_TYPE() IMPLEMENTATION





